@@ -367,6 +367,11 @@ class URLItem(object):
         '''Return the :class:`.database.URLRecord`.'''
         return self._url_record
 
+    @property
+    def url_table(self):
+        '''Return the :class:`.database.URLTable`.'''
+        return self._url_table
+
     @property
     def is_processed(self):
         '''Return whether the item has been processed.'''
